'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babylon = require('babylon');

var babylon = _interopRequireWildcard(_babylon);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _virtualJade = require('virtual-jade');

var _virtualJade2 = _interopRequireDefault(_virtualJade);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var fileExists = function fileExists(filename) {
  try {
    var stats = _fs2.default.statSync(filename);
    return stats.isFile(filename);
  } catch (e) {
    if (e.code === 'ENOENT') {
      return false;
    } else {
      throw Error(e);
    }
  }
};

var compileVirtualJadeFile = function compileVirtualJadeFile(jsFile, jadeFile) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  // try to resolve as file path
  var from = resolveModulePath(jsFile);
  var path = (0, _path.resolve)(from, jadeFile);
  if (!fileExists(path)) {
    // try to resolve from node modules
    path = (0, _path.resolve)('./node_modules', jadeFile);
  }

  if (!fileExists(path)) {
    throw Error('Cannot find jade file: ' + jadeFile);
  }

  var jadeContent = _fs2.default.readFileSync(path, 'utf8');
  return (0, _virtualJade2.default)(jadeContent, Object.assign({
    filename: path
  }, options));
};

var resolveModulePath = function resolveModulePath(filename) {
  var dir = (0, _path.dirname)(filename);
  if ((0, _path.isAbsolute)(dir)) return dir;
  if (process.env.PWD) return (0, _path.resolve)(process.env.PWD, dir);
  return (0, _path.resolve)(dir);
};

exports.default = function () {
  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path, state) {
        var node = path.node;
        if (node && node.source && node.source.value && node.source.type === 'StringLiteral' && node.source.value.endsWith('.jade')) {
          var jsFile = state.file.opts.filename;
          var jadeFile = node.source.value;
          var jadeFnStr = compileVirtualJadeFile(jsFile, jadeFile, state.opts);
          var varName = node.specifiers[0].local.name;
          var srcString = 'var ' + varName + ' = ' + jadeFnStr;
          path.replaceWith(babylon.parse(srcString));
        }
      }
    }
  };
};